@@ -383,7 +383,7 @@ static int parse_first_line(const char *buf, int *num, char **filename)
     *num = strtol(local_buf, NULL, 10);
 
     c++;
-    while(isspace(*c)) c++;
+    while (isspace(*c)) c++;
 
     *filename = nvstrdup(c);
 
@@ -678,7 +678,7 @@ static int do_uninstall(Options *op, const char *version,
 
     /* Remove any installed DKMS modules */
 
-    if (dkms_module_installed(op, version)) {
+    if (dkms_module_installed(op, version, NULL)) {
         ui_log(op, "DKMS module detected; removing...");
         if (!dkms_remove_module(op, version)) {
             ui_warn(op, "Failed to remove installed DKMS module!");
@@ -844,11 +844,11 @@ static int do_uninstall(Options *op, const char *version,
 
         if (!op->skip_depmod) {
             char *cmd = nvstrcat(op->utils[DEPMOD], " -a ", op->kernel_name, NULL);
-            status |= run_command(op, cmd, NULL, FALSE, 0, FALSE);
+            status |= run_command(op, cmd, NULL, FALSE, NULL, FALSE);
             nvfree(cmd);
         }
 
-        status |= run_command(op, op->utils[LDCONFIG], NULL, FALSE, 0, FALSE);
+        status |= run_command(op, op->utils[LDCONFIG], NULL, FALSE, NULL, FALSE);
 
         if (status == 0) {
             ui_log(op, "done.");
@@ -867,7 +867,7 @@ static int do_uninstall(Options *op, const char *version,
             char *cmd = nvstrcat(op->utils[SYSTEMCTL], " daemon-reload", NULL);
 
             ui_log(op, "Running `%s`:", cmd);
-            status = run_command(op, cmd, NULL, FALSE, 0, FALSE);
+            status = run_command(op, cmd, NULL, FALSE, NULL, FALSE);
             nvfree(cmd);
 
             if (status == 0) {
@@ -965,7 +965,7 @@ static BackupInfo *read_backup_log_file(Options *op)
     b->e = NULL;
     line_num = 3;
 
-    while(1) {
+    while (1) {
 
         percent = (float) (c - buf) / (float) stat_buf.st_size;
         ui_status_update(op, percent, NULL);
@@ -1302,9 +1302,9 @@ int get_installed_driver_version_and_descr(Options *op,
  * Returns FALSE if the user decided they didn't want to continue with
  * installation.
  *
- * If we are only installing a kernel module, then there must be an
+ * If we are only installing kernel modules, then there must be an
  * existing driver installation, and the version of that installation
- * must match the module we're trying to install.
+ * must match the modules we're trying to install.
  */
 
 int check_for_existing_driver(Options *op, Package *p)
@@ -1318,20 +1318,20 @@ int check_for_existing_driver(Options *op, Package *p)
 
     localRet = get_installed_driver_version_and_descr(op, &version, &descr);
 
-    if (op->kernel_module_only) {
+    if (op->kernel_modules_only) {
         if (!localRet) {
             ui_error(op, "No NVIDIA driver is currently installed; the "
-                     "'--kernel-module-only' option can only be used "
-                     "to install the NVIDIA kernel module on top of an "
+                     "'--kernel-modules-only' option can only be used "
+                     "to install the NVIDIA kernel modules on top of an "
                      "existing driver installation.");
             goto done;
         } else {
             if (strcmp(p->version, version) != 0) {
-                ui_error(op, "The '--kernel-module-only' option can only be "
-                         "used to install a kernel module on top of an "
+                ui_error(op, "The '--kernel-modules-only' option can only be "
+                         "used to install kernel modules on top of an "
                          "existing driver installation of the same driver "
                          "version.  The existing driver installation is "
-                         "%s, but the kernel module is %s.\n",
+                         "%s, but the kernel modules are %s.\n",
                          version, p->version);
                 goto done;
             } else {
@@ -1448,7 +1448,7 @@ static int check_skip_depmod_support(Options *op, const char *uninstaller)
     char *cmd = nvstrcat(uninstaller, " -A | ", op->utils[GREP],
                          " -q '^ \\+--skip-depmod$'", NULL);
 
-    int ret = run_command(op, cmd, NULL, FALSE, 0, FALSE);
+    int ret = run_command(op, cmd, NULL, FALSE, NULL, FALSE);
 
     nvfree(cmd);
 
@@ -1470,45 +1470,59 @@ int run_existing_uninstaller(Options *op)
      * kernel modules and run depmod afterwards, we don't need to run depmod
      * as part of uninstallation.
      */
-    int skip_depmod = !op->no_kernel_module;
+    int skip_depmod = !op->no_kernel_modules;
 
     if (uninstaller) {
+        char *uninstall_log_dir, *uninstall_log_file, *uninstall_log_path;
         char *uninstall_cmd = NULL;
         char *data = NULL;
         int ret;
 
         skip_depmod = skip_depmod && check_skip_depmod_support(op, uninstaller);
 
+        /*
+         * Use DEFAULT_UNINSTALL_LOG_FILE_NAME as the name for the uninstall
+         * log file, in the same directory as op->log_file_name, be it the
+         * default location or a custom one.
+         */
+        uninstall_log_dir = nv_dirname(op->log_file_name);
+        uninstall_log_file = nv_basename(DEFAULT_UNINSTALL_LOG_FILE_NAME);
+        uninstall_log_path = nvdircat(uninstall_log_dir, uninstall_log_file,
+                                      NULL);
+        nvfree(uninstall_log_dir);
+        nvfree(uninstall_log_file);
+
         /* Run the uninstaller non-interactively, and explicitly log to the
          * uninstall log location: older installers may not do so implicitly. */
-        uninstall_cmd = nvstrcat(uninstaller, " -s --log-file-name="
-                                 DEFAULT_UNINSTALL_LOG_FILE_NAME,
+        uninstall_cmd = nvstrcat(uninstaller, " -s --log-file-name=",
+                                 uninstall_log_path,
                                  skip_depmod ? " --skip-depmod" : NULL,
                                  NULL);
 
         ui_log(op, "Uninstalling the previous installation with %s.",
                uninstaller);
 
-        ret = run_command(op, uninstall_cmd, &data, FALSE, 0, TRUE);
+        ret = run_command(op, uninstall_cmd, &data, FALSE, NULL, TRUE);
 
         nvfree(uninstall_cmd);
 
         /* if nvidia-uninstall succeeded, return early; otherwise, fall back to
          * uninstalling via the backup log file. */
-        if (ret == 0) {
-            nvfree(uninstaller);
-            nvfree(data);
-            return TRUE;
-        } else {
+        if (ret != 0) {
             ui_log(op, "%s failed; see %s for more details.", uninstaller,
-                   DEFAULT_UNINSTALL_LOG_FILE_NAME);
+                   uninstall_log_path);
             if (data && strlen(data)) {
                 ui_log(op, "The output from %s was:\n%s", uninstaller, data);
             }
-            nvfree(data);
         }
 
+        nvfree(data);
         nvfree(uninstaller);
+        nvfree(uninstall_log_path);
+
+        if (ret == 0) {
+            return TRUE;
+        }
     }
 
     return uninstall_existing_driver(op, FALSE /* interactive */,
