@@ -34,6 +34,7 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <errno.h>
+#include <utime.h>
 
 #include "nvidia-installer.h"
 #include "command-list.h"
@@ -209,16 +210,18 @@ CommandList *build_command_list(Options *op, Package *p)
 
     /* find any possibly conflicting modules and/or libraries */
 
-    if (!op->no_kernel_module || op->dkms) {
+    if (!op->no_kernel_modules && !op->dkms_registered) {
         find_conflicting_kernel_modules(op, l);
     }
 
     /* check the conflicting file list for any installed kernel modules */
 
-    if (op->kernel_module_only) {
-        if (dkms_module_installed(op, p->version)) {
-            ui_error(op, "A DKMS kernel module with version %s is already "
-                     "installed.", p->version);
+    if (op->kernel_modules_only) {
+        const char *kernel = get_kernel_name(op);
+
+        if (dkms_module_installed(op, p->version, kernel)) {
+            ui_error(op, "DKMS kernel modules with version %s are already "
+                     "installed for the %s kernel.", p->version, kernel);
             free_file_list(l);
             free_command_list(op,c);
             return NULL;
@@ -234,16 +237,16 @@ CommandList *build_command_list(Options *op, Package *p)
             }
         }
 
-        /* XXX: If installing with --kernel-module-only on a system that has
-         * the kernel module sources already installed, but does NOT have a
-         * built kernel module or DKMS module, duplicate entries for the source
-         * files will be added to the backup log, leading to error messages
-         * when uninstalling the driver later leads to redundant attempts to
-         * delete the files. */
+        /* XXX: If installing with --kernel-modules-only on a system that has
+         * the kernel module sources already installed, but does NOT have built
+         * kernel modules or DKMS modules, duplicate entries for the source
+         * files will be added to the backup log, leading to error messages when
+         * uninstalling the driver later leads to redundant attempts to delete
+         * the files. */
 
     }
     
-    if (!op->kernel_module_only) {
+    if (!op->kernel_modules_only) {
         char **paths;
         int numpaths, i;
         ConflictingFileInfo *conflicting_files;
@@ -288,6 +291,17 @@ CommandList *build_command_list(Options *op, Package *p)
 
     tmp_installable_files = installable_files;
     add_symlinks_to_file_type_list(&tmp_installable_files);
+    if (op->dkms_registered) {
+        /*
+         * These files were imported from the DKMS tarball and DKMS will remove
+         * them when the kernel modules are `dkms remove`d, so nvidia-installer
+         * should not install them over the existing copies.
+         */
+        remove_file_type_from_file_type_list(&tmp_installable_files,
+                                             FILE_TYPE_DKMS_CONF);
+        remove_file_type_from_file_type_list(&tmp_installable_files,
+                                             FILE_TYPE_KERNEL_MODULE_SRC);
+    }
 
     find_existing_files(p, l, &tmp_installable_files);
     
@@ -342,6 +356,17 @@ CommandList *build_command_list(Options *op, Package *p)
 
         nvfree(tmp);
 
+        /*
+         * For icons, update the mtime on the top-level directory of the icon
+         * theme. This triggers desktop environments to invalidate their icon
+         * cache.
+         *
+         * See https://specifications.freedesktop.org/icon-theme-spec/icon-theme-spec-latest.html#implementation_notes
+         */
+        if (p->entries[i].type == FILE_TYPE_ICON) {
+            add_command(c, TOUCH_CMD, op->icon_dir);
+        }
+
         /*
          * delete any temporary generated files
          */
@@ -408,7 +433,7 @@ CommandList *build_command_list(Options *op, Package *p)
      * <Nigel.Spowage@energis.com>
      */
     
-    if (!op->no_kernel_module && !op->skip_depmod) {
+    if (!op->no_kernel_modules && !op->skip_depmod) {
         tmp = nvstrcat(op->utils[DEPMOD], " -a ", op->kernel_name, NULL);
         add_command(c, RUN_CMD, tmp);
         nvfree(tmp);
@@ -489,7 +514,7 @@ static inline int execute_run_command(Options *op, float percent, const char *cm
     ui_expert(op, "Executing: %s", cmd);
     ui_status_update(op, percent, "Executing: `%s` "
                      "(this may take a moment...)", cmd);
-    ret = run_command(op, cmd, &data, TRUE, 0, TRUE);
+    ret = run_command(op, cmd, &data, TRUE, NULL, TRUE);
     if (ret != 0) {
         ui_error(op, "Failed to execute `%s`: %s", cmd, data);
         ret = continue_after_error(op, "Failed to execute `%s`", cmd);
@@ -592,6 +617,16 @@ int execute_command_list(Options *op, CommandList *c,
             }
             break;
 
+        case TOUCH_CMD:
+            ui_expert(op, "Updating mtime: %s", c->cmds[i].s0);
+            ret = utime(c->cmds[i].s0, NULL);
+            if (ret == -1) {
+                ret = continue_after_error(op, "Cannot touch %s",
+                                           c->cmds[i].s0);
+                if (!ret) return FALSE;
+            }
+            break;
+
         default:
             /* XXX should never get here */
             return FALSE;
@@ -1052,6 +1087,7 @@ static void add_command(CommandList *c, int cmd, ...)
         c->cmds[n].s1 = nvstrdup(s);
         break;
       case DELETE_CMD:
+      case TOUCH_CMD:
         s = va_arg(ap, char *);
         c->cmds[n].s0 = nvstrdup(s);
         break;
