@@ -97,12 +97,15 @@ int install_from_cwd(Options *op)
 {
     Package *p;
     CommandList *c;
-    int ret;
     int ran_pre_install_hook = FALSE;
     HookScriptStatus res;
 
-    static const char* edit_your_xf86config =
-        "Please update your xorg.conf file as "
+    static const char* module_only_text =
+        "kernel module for the ";
+    static const char* xconfig_success_text =
+        "Your X configuration file has been successfully updated.  ";
+    static const char* edit_your_xorgconf_text =
+        "  Please update your xorg.conf file as "
         "appropriate; see the file /usr/share/doc/"
         "NVIDIA_GLX-1.0/README.txt for details.";
 
@@ -114,7 +117,7 @@ int install_from_cwd(Options *op)
     if ((p = parse_manifest(op)) == NULL) goto failed;
 
     if (!op->x_files_packaged) {
-        edit_your_xf86config = "";
+        edit_your_xorgconf_text = "";
     }
 
     ui_set_title(op, "%s (%s)", p->description, p->version);
@@ -198,37 +201,28 @@ int install_from_cwd(Options *op)
 
     /* attempt to build the kernel modules for the target kernel */
 
-    if (!op->no_kernel_module) {
+    if (!op->no_kernel_modules) {
         if (!install_kernel_modules(op, p)) {
             goto failed;
         }
     } else {
-        ui_warn(op, "You specified the '--no-kernel-module' command line "
-                "option, nvidia-installer will not install a kernel "
-                "module as part of this driver installation, and it will "
+        ui_warn(op, "You specified the '--no-kernel-modules' command line "
+                "option, nvidia-installer will not install any kernel "
+                "modules as part of this driver installation, and it will "
                 "not remove existing NVIDIA kernel modules not part of "
                 "an earlier NVIDIA driver installation.  Please ensure "
-                "that an NVIDIA kernel module matching this driver version "
-                "is installed separately.");
-
-        /* no_kernel_module should imply no DKMS */
-
-        if (op->dkms) {
-            ui_warn(op, "You have specified both the '--no-kernel-module' "
-                    "and the '--dkms' command line options. The '--dkms' "
-                    "option will be ignored.");
-            op->dkms = FALSE;
-        }
+                "that NVIDIA kernel modules matching this driver version "
+                "are installed separately.");
     }
     
     /*
-     * if we are only installing the kernel module, then remove
+     * if we are only installing the kernel modules, then remove
      * everything else from the package; otherwise do some
      * OpenGL-specific stuff
      */
 
-    if (op->kernel_module_only) {
-        remove_non_kernel_module_files_from_package(op, p);
+    if (op->kernel_modules_only) {
+        remove_non_kernel_module_files_from_package(p);
     } else {
 
         /* ask for the XFree86 and OpenGL installation prefixes. */
@@ -255,20 +249,27 @@ int install_from_cwd(Options *op)
     }
 
     if (op->no_opengl_files) {
-        remove_opengl_files_from_package(op, p);
+        remove_opengl_files_from_package(p);
+    } else {
+        check_for_vulkan_loader(op);
     }
 
     if (op->no_wine_files) {
-        remove_wine_files_from_package(op, p);
+        remove_wine_files_from_package(p);
     }
 
     /*
      * determine whether systemd files should be installed
      */
     if (op->use_systemd != NV_OPTIONAL_BOOL_TRUE) {
-        remove_systemd_files_from_package(op, p);
+        remove_systemd_files_from_package(p);
     }
 
+    /*
+     * Remove any kernel module source files that won't be installed.
+     */
+    remove_non_installed_kernel_module_source_files_from_package(p);
+
     /*
      * now that we have the installation prefixes, build the
      * destination for each file to be installed
@@ -281,22 +282,29 @@ int install_from_cwd(Options *op)
      * installed to /usr/lib/libGL.so.1. add_libgl_abi_symlink() sets its own
      * destination, so it must be called after set_destinations().
      */
-    if (!op->kernel_module_only && !op->no_opengl_files) {
+    if (!op->kernel_modules_only && !op->no_opengl_files) {
         add_libgl_abi_symlink(op, p);
     }
-    
-    /*
-     * uninstall the existing driver; this needs to be done before
-     * building the command list.
-     *
-     * XXX if we uninstall now, then build the command list, and
-     * then ask the user if they really want to execute the
-     * command list, if the user decides not to execute the
-     * command list, they'll be left with no driver installed.
-     */
 
-    if (!op->kernel_module_only) {
+    if (!op->kernel_modules_only) {
+        /*
+         * uninstall the existing driver; this needs to be done before
+         * building the command list.
+         *
+         * XXX if we uninstall now, then build the command list, and
+         * then ask the user if they really want to execute the
+         * command list, if the user decides not to execute the
+         * command list, they'll be left with no driver installed.
+         */
         if (!run_existing_uninstaller(op)) goto failed;
+
+        /* initialize the backup log */
+        if (!init_backup(op, p)) goto failed;
+    }
+
+    if (!op->no_kernel_modules) {
+        /*  Import the modules into DKMS, if requested */
+        dkms_register_module(op, p, get_kernel_name(op));
     }
 
     if (!check_libglvnd_files(op, p)) {
@@ -312,28 +320,17 @@ int install_from_cwd(Options *op)
     if (!ui_approve_command_list(op, c, "%s", p->description)) {
         goto exit_install;
     }
-    
-    /* initialize the backup log file */
-
-    if (!op->kernel_module_only) {
-        if (!init_backup(op, p)) goto failed;
-    }
 
     /* execute the command list */
 
     if (!do_install(op, p, c)) goto failed;
 
-    /* Register, build, and install the module with DKMS, if requested */
-
-    if (op->dkms && !dkms_install_module(op, p->version, get_kernel_name(op)))
-        goto failed;
-
     /*
      * Leave nvidia-drm loaded in case an X server with OutputClass-based driver
      * matching is being used.
      */
 
-    if (!op->no_kernel_module || op->dkms) {
+    if (!op->no_kernel_modules) {
         if (package_includes_kernel_module(p, "nvidia-drm")) {
             if (!load_kernel_module(op, "nvidia-drm")) {
                 goto failed;
@@ -358,37 +355,40 @@ int install_from_cwd(Options *op)
 
     check_installed_files_from_package(op, p);
 
-    if (!check_runtime_configuration(op, p)) goto failed;
-    
     /* done */
 
-    if (op->kernel_module_only || op->no_nvidia_xconfig_question) {
+    if (!op->kernel_modules_only) {
+        module_only_text = "";
+    }
 
-        ui_message(op, "Installation of the kernel module for the %s "
-                   "(version %s) is now complete.",
-                   p->description, p->version);
+    if (op->kernel_modules_only || op->no_nvidia_xconfig_question) {
+        xconfig_success_text = "";
+        edit_your_xorgconf_text = "";
     } else {
-        
+        int ret;
+
         /* ask the user if they would like to run nvidia-xconfig */
-        
+
         const char *msg = "Would you like to run the nvidia-xconfig utility "
                           "to automatically update your X configuration file "
                           "so that the NVIDIA X driver will be used when you "
                           "restart X?  Any pre-existing X configuration "
                           "file will be backed up.";
-        
+
         ret = run_nvidia_xconfig(op, FALSE, msg, op->run_nvidia_xconfig);
         
         if (ret) {
-            ui_message(op, "Your X configuration file has been successfully "
-                       "updated.  Installation of the %s (version: %s) is now "
-                       "complete.", p->description, p->version);
+            edit_your_xorgconf_text = "";
         } else {
-            ui_message(op, "Installation of the %s (version: %s) is now "
-                       "complete.  %s", p->description,
-                       p->version, edit_your_xf86config);
+            xconfig_success_text = "";
         }
     }
+
+    ui_message(op,
+               "%sInstallation of the %s%s (version: %s) is now complete.%s",
+               xconfig_success_text, module_only_text,
+               p->description, p->version,
+               edit_your_xorgconf_text);
     
     free_package(p);
 
@@ -451,26 +451,6 @@ static int install_kernel_modules(Options *op,  Package *p)
 
     process_dkms_conf(op,p);
 
-    /* Offer the DKMS option if DKMS exists and the kernel module sources 
-     * will be installed somewhere. Don't offer DKMS as an option if module
-     * signing was requested. */
-
-    if (op->utils[DKMS] && !op->no_kernel_module_source &&
-        !(op->module_signing_secret_key && op->module_signing_public_key)) {
-        op->dkms = ui_yes_no(op, op->dkms,
-                            "Would you like to register the kernel module "
-                            "sources with DKMS? This will allow DKMS to "
-                            "automatically build a new module, if you "
-                            "install a different kernel later.");
-    }
-
-    /* Only do the normal kernel module install if not using DKMS */
-
-    if (op->dkms) {
-        op->no_kernel_module = TRUE;
-        return TRUE;
-    }
-
     /* determine where to install the kernel module */
     
     if (!determine_kernel_module_installation_path(op)) return FALSE;
@@ -897,6 +877,9 @@ static Package *parse_manifest (Options *op)
             case FILE_TYPE_XMODULE_SHARED_LIB:
                 op->x_files_packaged = TRUE;
                 break;
+            case FILE_TYPE_VULKAN_ICD_JSON:
+                op->vulkan_icd_json_packaged = TRUE;
+                break;
             default: break;
         }
 
@@ -1121,6 +1104,8 @@ static void free_package(Package *p)
 
     nvfree(p->excluded_kernel_modules);
 
+    nvfree(p->kernel_make_logs);
+
     for (i = 0; i < p->num_entries; i++) {
         nvfree(p->entries[i].file);
         nvfree(p->entries[i].path);
@@ -1238,6 +1223,10 @@ static int assisted_module_signing(Options *op, Package *p)
         if (generate_keys) {
             char *cmdline, *x509_hash, *private_key_path, *public_key_path;
             int ret, generate_failed = FALSE;
+            const RunCommandOutputMatch output_match[] = {
+                { .lines = 8, .initial_match = NULL },
+                { 0 }
+            };
 
             if (!op->utils[OPENSSL]) {
                 ui_error(op, "Unable to generate key pair: openssl not "
@@ -1329,7 +1318,7 @@ guess_fail:
                                " -", x509_hash, NULL);
             nvfree(x509_hash);
 
-            ret = run_command(op, cmdline, NULL, TRUE, 8, TRUE);
+            ret = run_command(op, cmdline, NULL, TRUE, output_match, TRUE);
 
             nvfree(cmdline);
 
@@ -1394,7 +1383,7 @@ generate_done:
         cmdline = nvstrcat(op->utils[OPENSSL], " x509 -noout -fingerprint ",
                            "-inform DER -in ", op->module_signing_public_key,
                            NULL);
-        ret = run_command(op, cmdline, &result, FALSE, 0, FALSE);
+        ret = run_command(op, cmdline, &result, FALSE, NULL, FALSE);
         nvfree(cmdline);
 
         /* Format: "SHA1 Fingerprint=00:00:00:00:..." */
@@ -1408,7 +1397,7 @@ generate_done:
                  * incorrectly; try to get a sha1sum of the DER certificate */
                 cmdline = nvstrcat(sha1sum, " ", op->module_signing_public_key,
                                    NULL);
-                ret = run_command(op, cmdline, &result, FALSE, 0, FALSE);
+                ret = run_command(op, cmdline, &result, FALSE, NULL, FALSE);
                 nvfree(sha1sum);
                 nvfree(cmdline);
 
@@ -1426,7 +1415,7 @@ generate_done:
         } else {
             /* Remove any ':' characters from fingerprint and truncate */
             char *tmp = nv_strreplace(fingerprint, ":", "");
-            strncpy(short_fingerprint, tmp, sizeof(short_fingerprint));
+            strncpy(short_fingerprint, tmp, sizeof(short_fingerprint) - 1);
             nvfree(tmp);
         }
         short_fingerprint[sizeof(short_fingerprint) - 1] = '\0';
