@@ -57,16 +57,15 @@ static PrecompiledInfo *scan_dir(Options *op, Package *p,
 
 static char *build_distro_precompiled_kernel_interface_dir(Options *op);
 static char *convert_include_path_to_source_path(const char *inc);
-static char *get_machine_arch(Options *op);
 static int run_conftest(Options *op, const char *dir, const char *args,
                         char **result);
-static int run_make(Options *op, Package *p, const char *dir, const char *target,
-                    char **vars, const char *status, int lines);
+static int run_make(Options *op, Package *p, const char *dir,
+                    const char *cli_options, const char *status,
+                    const RunCommandOutputMatch *match);
 static void load_kernel_module_quiet(Options *op, const char *module_name);
 static void modprobe_remove_kernel_module_quiet(Options *op, const char *name);
 static int kernel_configuration_conflict(Options *op, Package *p,
                                          int target_system_checks);
-static int check_cc_version(Options *op, Package *p);
 
 /*
  * Message text that is used by several error messages.
@@ -156,7 +155,8 @@ int determine_kernel_module_installation_path(Options *op)
 static int run_conftest(Options *op, const char *dir, const char *args,
                         char **result)
 {
-    char *cmd, *arch, *kernel_source_path, *kernel_output_path;
+    char *cmd, *kernel_source_path, *kernel_output_path;
+    const char *arch;
     int ret;
 
     if (result) {
@@ -306,24 +306,22 @@ int determine_kernel_source_path(Options *op, Package *p)
 
     ret = access(version_h, F_OK) == 0 || access(uapi_version_h, F_OK) == 0;
 
-    free(version_h);
-    free(uapi_version_h);
-
-    if (!ret) {
-        ui_error(op, "Neither the '" VERSION_H_PATH "' nor the '"
-                 UAPI_VERSION_H_PATH "' kernel header file exists.  "
+    if (ret) {
+        /* OK, we seem to have a path to a configured kernel source tree */
+    
+        ui_log(op, "Kernel source path: '%s'\n", op->kernel_source_path);
+        ui_log(op, "Kernel output path: '%s'\n", op->kernel_output_path);
+    } else {
+        ui_error(op, "Neither the '%s' nor the '%s' kernel header file exists. "
                  "The most likely reason for this is that the kernel "
                  "source files in '%s' have not been configured.",
-                 op->kernel_output_path);
-        return FALSE;
+                 version_h, uapi_version_h, op->kernel_output_path);
     }
 
-    /* OK, we seem to have a path to a configured kernel source tree */
+    free(version_h);
+    free(uapi_version_h);
     
-    ui_log(op, "Kernel source path: '%s'\n", op->kernel_source_path);
-    ui_log(op, "Kernel output path: '%s'\n", op->kernel_output_path);
-    
-    return TRUE;
+    return ret;
     
 } /* determine_kernel_source_path() */
 
@@ -337,7 +335,6 @@ int determine_kernel_source_path(Options *op, Package *p)
 int determine_kernel_output_path(Options *op)
 {
     char *str, *tmp;
-    int len;
 
     /* check --kernel-output-path */
 
@@ -379,11 +376,19 @@ int determine_kernel_output_path(Options *op)
     tmp = get_kernel_name(op);
 
     if (tmp) {
-        str = nvstrcat("/lib/modules/", tmp, "/source", NULL);
-        len = strlen(str);
+        char *source_path, *build_source_path;
+        int len_source_path, len_build_source_path;
 
-        if (!strncmp(op->kernel_source_path, str, len)) {
-            nvfree(str);
+        source_path = nvstrcat("/lib/modules/", tmp, "/source", NULL);
+        len_source_path = strlen(source_path);
+
+        build_source_path = nvstrcat("/lib/modules/", tmp, "/build/source", NULL);
+        len_build_source_path = strlen(build_source_path);
+
+        if ((!strncmp(op->kernel_source_path, source_path, len_source_path)) ||
+            (!strncmp(op->kernel_source_path, build_source_path, len_build_source_path))) {
+            nvfree(source_path);
+            nvfree(build_source_path);
             str = nvstrcat("/lib/modules/", tmp, "/build", NULL);
 
             if (directory_exists(str)) {
@@ -391,7 +396,10 @@ int determine_kernel_output_path(Options *op)
                 return TRUE;
             }
         }
-
+        else  {
+            nvfree(source_path);
+            nvfree(build_source_path);
+        }
         nvfree(str);
     }
 
@@ -467,7 +475,7 @@ static int attach_signature(Options *op, Package *p,
     }
 
     if (ret) {
-        if(op->kernel_module_signed) {
+        if (op->kernel_module_signed) {
             ui_log(op, "Signature attached successfully.");
         } else {
             ui_log(op, "Signature not attached.");
@@ -545,6 +553,84 @@ int unpack_kernel_modules(Options *op, Package *p, const char *build_directory,
    
 }
 
+/*
+ * Estimate the number of expected lines of output that will be produced by
+ * building the kernel modules. single_module may be set to restrict the
+ * estimate to the result of building the specified module only.
+ */
+static RunCommandOutputMatch *count_lines(Options *op, Package *p,
+                                          const char *dir,
+                                          const char *single_module)
+{
+    RunCommandOutputMatch *ret = nvalloc(sizeof(*ret) * 5);
+    int conftest_count, object_count, module_count, count_success = FALSE;
+    char *data = NULL, *cmd;
+
+    /*
+     * Build the make(1) command line. run_make() is explicitly avoided here:
+     * the output from count-lines.mk shouldn't be logged or displayed.
+     */
+    cmd = nvstrcat("cd ", dir, "; ",
+                   op->utils[MAKE], " -f count-lines.mk count "
+                   "NV_EXCLUDE_KERNEL_MODULES=", p->excluded_kernel_modules,
+                   single_module ? "" : NULL,
+                   " NV_KERNEL_MODULES=", single_module,
+                   NULL);
+
+    if (run_command(op, cmd, &data, FALSE, NULL, TRUE) == 0) {
+        if (sscanf(data, "conftests:%d objects:%d modules:%d",
+                   &conftest_count, &object_count, &module_count) == 3) {
+            count_success = TRUE;
+        }
+    }
+
+    if (!count_success) {
+        /*
+         * Something went wrong, but since the counts are only used for cosmetic
+         * purposes, it is sufficient to log the error and silently fall back to
+         * approximate default values.
+         */
+        ui_log(op, "Failed to estimate output lines: %s", data);
+        conftest_count = 250; object_count = 200; module_count = 5;
+    }
+
+    /*
+     * One line each for entering and leaving each of the Kbuild source and
+     * output directories
+     */
+    ret[0].lines = 4;
+    ret[0].initial_match = "make[";
+
+    /*
+     * Each C source file that is compiled into an object file generates one
+     * line of output beginning with "  CC [M] ". Additionally, for each module
+     * a $module_name.mod.o file is compiled, outputting a line that begins with
+     * "  CC " on older kernels and "  CC [M] " on newer ones.
+     */
+    ret[1].lines = object_count + module_count;
+    ret[1].initial_match = "  CC ";
+
+    /*
+     * Each module has has a $module_name.o object linked before the MODPOST
+     * stage, and a $module_name.ko object linked after. For both linking steps,
+     * a line beginning with "  LD [M] " is printed.
+     */
+    ret[2].lines = module_count * 2;
+    ret[2].initial_match = "  LD [M] ";
+
+    /*
+     * Expect one line of output per conftest, and assume that all the conftests
+     * have already been run in the "rebuild a single module to isolate modules
+     * that failed to build" case.
+     */
+    if (single_module == NULL) {
+        ret[3].lines = conftest_count;
+        ret[3].initial_match = " CONFTEST: ";
+    }
+
+    return ret;
+}
+
 
 static int check_file(Options *op, Package *p, const char *dir,
                       const char *modname)
@@ -561,11 +647,13 @@ static int check_file(Options *op, Package *p, const char *dir,
         char *rebuild_msg = nvstrcat("Checking to see whether the ", modname,
                                      " kernel module was successfully built",
                                      NULL);
+        RunCommandOutputMatch *match = count_lines(op, p, dir, modname);
         /* Attempt to rebuild the individual module, in case the failure
          * is module-specific and due to a different module */
-        run_make(op, p, dir, single_module_list, NULL, rebuild_msg, 25);
+        run_make(op, p, dir, single_module_list, rebuild_msg, match);
         nvfree(single_module_list);
         nvfree(rebuild_msg);
+        nvfree(match);
 
         /* Check the file again */
         ret = access(path, F_OK);
@@ -592,33 +680,6 @@ int build_kernel_modules(Options *op, Package *p)
 }
 
 
-/*
- * Run the module_signing_script with three or four arguments.
- * Return TRUE on success.
- */
-static int try_sign_file(Options *op, const char *file, int num_args)
-{
-    char *cmd;
-    const char *arg_1;
-    int ret;
-
-    switch (num_args) {
-        case 3: arg_1 = ""; break;
-        case 4: arg_1 = op->module_signing_hash; break;
-        default: return FALSE;
-    }
-
-    cmd = nvstrcat("\"", op->module_signing_script, "\" ",
-                   arg_1, " \"",
-                   op->module_signing_secret_key, "\" \"",
-                   op->module_signing_public_key, "\" \"",
-                   file, "\"", NULL);
-    ret = run_command(op, cmd, NULL, TRUE, 1, TRUE);
-    nvfree(cmd);
-
-    return ret == 0;
-}
-
 
 /*
  * Check to see if scripts/sign-file exists under 'dir' and is executable
@@ -646,9 +707,12 @@ static char *test_sign_file(const char *dir)
  */
 int sign_kernel_module(Options *op, const char *build_directory, 
                        const char *module_filename, int status) {
-    char *file;
+    const RunCommandOutputMatch output_match[] = {
+        { .lines = 1, .initial_match = NULL },
+        { 0 }
+    };
     int success;
-    static int num_args = 3;
+    char *cmd;
 
     /* Lazily set the default value for module_signing_script. */
 
@@ -672,39 +736,27 @@ int sign_kernel_module(Options *op, const char *build_directory,
         ui_status_begin(op, "Signing kernel module:", "Signing");
     }
 
-    file = nvstrcat(build_directory, "/", module_filename, NULL);
-
-  try_sign:
-
-    success = try_sign_file(op, file, num_args);
-
-    /* If sign-file failed to run with three arguments, try running it with
-     * four arguments on all subsequent signing attempts. */
-
-    if (num_args == 3 && !success) {
-        num_args = 4;
-
-        /* The four-arg version of sign-file needs a hash. */
-
-        if (!op->module_signing_hash) {
-            op->module_signing_hash = guess_module_signing_hash(op,
-                                          build_directory);
-        }
-
-        if (op->module_signing_hash) {
-            goto try_sign;
-        } else {
-            ui_error(op, "The installer was unable to sign %s without "
-                     "specifying a hash algorithm on the command line to %s, "
-                     "and was also unable to automatically detect the hash. "
-                     "If you need to sign the NVIDIA kernel modules, please "
-                     "try again and set the '--module-signing-hash' option on "
-                     "the installer's command line.",
-                     file, op->module_signing_script);
-        }
+    if (!op->module_signing_hash) {
+        op->module_signing_hash = guess_module_signing_hash(op,
+                                                            build_directory);
     }
 
-    nvfree(file);
+    if (!op->module_signing_hash) {
+        ui_error(op, "The installer cannot sign %s without specifying a hash "
+                 "algorithm on the command line to %s, and was also unable to "
+                 "automatically detect the hash. If you need to sign the "
+                 "NVIDIA kernel modules, please try again and set the "
+                 "'--module-signing-hash' option on the installer's command "
+                 "line.", module_filename, op->module_signing_script);
+    }
+
+    cmd = nvstrcat("\"", op->module_signing_script, "\" ",
+                   op->module_signing_hash, " \"",
+                   op->module_signing_secret_key, "\" \"",
+                   op->module_signing_public_key, "\" \"",
+                   build_directory, "/", module_filename, "\"", NULL);
+    success = (run_command(op, cmd, NULL, TRUE, output_match, TRUE) == 0);
+    nvfree(cmd);
 
     if (status) {
         ui_status_end(op, success ? "done." : "Failed to sign kernel module.");
@@ -895,6 +947,7 @@ int build_kernel_interfaces(Options *op, Package *p,
 {
     char *tmpdir = NULL, *builddir;
     int ret, files_packaged = 0, i;
+    RunCommandOutputMatch *match;
 
     struct {
         const char *sanity_check_name;
@@ -957,15 +1010,12 @@ int build_kernel_interfaces(Options *op, Package *p,
         }
     }
 
-    /* run cc_version_check separately to allow check_cc_version() to set
-     * IGNORE_CC_MISMATCH if needed */
-    if (!check_cc_version(op, p)) {
-        return FALSE;
-    }
-
     ui_log(op, "Cleaning kernel module build directory.");
-    run_make(op, p, builddir, "clean", NULL, NULL, 0);
-    ret = run_make(op, p, builddir, "", NULL, "Building kernel modules", 25);
+    run_make(op, p, builddir, "clean", NULL, 0);
+
+    match = count_lines(op, p, builddir, NULL);
+    ret = run_make(op, p, builddir, "", "Building kernel modules", match);
+    nvfree(match);
 
     /* Test to make sure that all kernel modules were built. */
     for (i = 0; i < p->num_kernel_modules; i++) {
@@ -1006,7 +1056,7 @@ int build_kernel_interfaces(Options *op, Package *p,
 
         if (module->has_separate_interface_file) {
             if (!(run_make(op, p, tmpdir, module->interface_filename,
-                           NULL, NULL, 0) &&
+                           NULL, NULL) &&
                 pack_kernel_interface(op, p, tmpdir, fileInfo,
                                       module->interface_filename,
                                       module->module_filename,
@@ -1233,7 +1283,7 @@ static int ignore_load_error(Options *op, Package *p,
                                                probable_reason,
                                                signature_related) == 0);
         } else {
-            const char *secureboot_message, *dkms_message;
+            const char *secureboot_message;
 
             secureboot_message = secureboot == 1 ?
                                      "and sign the kernel module when "
@@ -1245,14 +1295,10 @@ static int ignore_load_error(Options *op, Package *p,
                                      "enable the interactive module "
                                      "signing prompts.";
 
-            dkms_message = op->dkms ? " Module signing is incompatible "
-                                      "with DKMS, so please select the "
-                                      "non-DKMS option when building the "
-                                      "kernel module to be signed." : "";
             ui_error(op, "The kernel module failed to load%s because it "
                      "was not signed by a key that is trusted by the "
-                     "kernel. Please try installing the driver again, %s%s",
-                     probable_reason, secureboot_message, dkms_message);
+                     "kernel. Please try installing the driver again, %s",
+                     probable_reason, secureboot_message);
         }
     }
 
@@ -1268,8 +1314,8 @@ static int ignore_load_error(Options *op, Package *p,
                  "from the one used to build the target kernel, or "
                  "if another driver, such as nouveau, is "
                  "present and prevents the NVIDIA kernel module from "
-                 "obtaining ownership of the NVIDIA GPU(s), "
-                 "or no NVIDIA GPU installed in this system is supported "
+                 "obtaining ownership of the NVIDIA device(s), "
+                 "or no NVIDIA device installed in this system is supported "
                  "by this NVIDIA Linux graphics driver release.\n\n"
                  "Please see the log entries 'Kernel module load "
                  "error' and 'Kernel messages' at the end of the file "
@@ -1303,16 +1349,74 @@ static void unload_kernel_modules(Options *op, Package *p) {
     }
 }
 
-
 /*
- * test_kernel_module() - attempt to insmod the kernel modules and then rmmod
- * them.  Return TRUE if the insmod succeeded, or FALSE otherwise.
+ * Enable or disable the udev event queue
  */
 
-int test_kernel_modules(Options *op, Package *p)
+static void toggle_udev_event_queue(Options *op, int enable)
+{
+    const char *verb  = enable ? "start" : "stop";
+    char *udevadm = find_system_util("udevadm");
+    static int already_warned = FALSE;
+
+    if (udevadm) {
+        /*
+         * We want to wait for udevadm(8) to finish disabling the event queue,
+         * but since enabling it again can take several seconds as the queue
+         * backlog is processed, set the timeout to 0 when restarting the queue
+         * so udevadm(8) can return right away.
+         */
+        const char *timeout = enable ? " --timeout=0" : NULL;
+        char *cmd, *data;
+        int cmd_ret;
+
+        cmd = nvstrcat(udevadm, " control --", verb, "-exec-queue", timeout,
+                       NULL);
+        nvfree(udevadm);
+
+        cmd_ret = run_command(op, cmd, &data, FALSE, NULL, TRUE) != 0;
+        nvfree(cmd);
+
+        if (cmd_ret != 0) {
+            ui_warn(op, "Failed to %s the udev event queue:\n\n%s",
+                    verb, data);
+        }
+        nvfree(data);
+    } else if (!already_warned) {
+        ui_warn(op, "Failed to find udevadm(8); nvidia-installer will not "
+                "be able to %s the udev event queue.", verb);
+        already_warned = TRUE;
+    }
+}
+
+/*
+ * log_dmesg(): display/log the last few lines of the kernel ring buffer to
+ * provide further details in case of a load failure or to capture NVRM warning
+ * messages, if any.
+ */
+static void log_dmesg(Options *op)
 {
     char *cmd = NULL, *data = NULL;
-    int ret = FALSE, i;
+
+    cmd = nvstrcat(op->utils[DMESG], " | ",
+                   op->utils[TAIL], " -n 25", NULL);
+
+    if (!run_command(op, cmd, &data, FALSE, NULL, TRUE))
+        ui_log(op, "Kernel messages:\n%s", data);
+
+    nvfree(cmd);
+    nvfree(data);
+}
+
+/*
+ * test_kernel_modules_helper(): test-load the kernel modules, optionally
+ * pausing the udev event queue. Passes the return value of do_insmod() back
+ * up to the caller, stopping early if do_insmod() fails for an individual
+ * kernel module.
+ */
+static int test_kernel_modules_helper(Options *op, Package *p, int pause_udev)
+{
+    int insmod_ret = -1, i;
     const char *depmods[] = {
         "i2c-core",
         "drm",
@@ -1322,24 +1426,46 @@ int test_kernel_modules(Options *op, Package *p)
         "mdev",
         "video",
         "backlight",
+        "vfio_pci_core",
+        "ecc",
     };
 
-    /* 
-     * If we're building/installing for a different kernel, then we
-     * can't test the module now.
-     */
-
-    if (op->kernel_name) return TRUE;
+    if (pause_udev) {
+        /*
+         * Temporarily disable the udev event queue to prevent modules from
+         * being automatically loaded by udev rules.
+         */
+        toggle_udev_event_queue(op, FALSE);
+    }
 
     /*
-     * Attempt to load modules that nvidia.ko might depend on.  Silently ignore
-     * failures: if nvidia.ko doesn't depend on the module that failed, the test
-     * load below will succeed and it doesn't matter that the load here failed.
+     * Attempt to load modules that the NVIDIA kernel modules might depend on.
      */
     for (i = 0; i < ARRAY_LEN(depmods); i++) {
-        load_kernel_module_quiet(op, depmods[i]);
+        if (check_for_loaded_kernel_module(op, depmods[i])) {
+            /*
+             * The dependency kernel module is already loaded: don't attempt to
+             * unload it later.
+             */
+            depmods[i] = NULL;
+        } else {
+            /*
+             * Silently ignore failures: the test load of the NVIDIA kernel
+             * modules may succeed anyway (e.g., if the relevant code is built
+             * into the kernel rather than a module.), and it won't matter that
+             * we failed to load one of the dependency modules.
+             */
+            load_kernel_module_quiet(op, depmods[i]);
+        }
     }
 
+    /*
+     * It's possible that one or more (existing) kernel modules may have been
+     * loaded between the initial attempt to unload the kernel modules and now;
+     * unload everything again just in case.
+     */
+    unload_kernel_modules(op, p);
+
     /*
      * Attempt to load each kernel module one at a time. The order of the list
      * in the package manifest is set such that loading modules in that order
@@ -1347,7 +1473,6 @@ int test_kernel_modules(Options *op, Package *p)
      */
 
     for (i = 0; i < p->num_kernel_modules; i++) {
-        int module_success = FALSE, insmod_ret;
         const char *module_opts = "";
         char *module_path;
 
@@ -1371,23 +1496,24 @@ int test_kernel_modules(Options *op, Package *p)
         insmod_ret = do_insmod(op, module_path, module_opts);
         nvfree(module_path);
 
-        if (insmod_ret == 0) {
-            module_success = TRUE;
-        } else {
-            ret = ignore_load_error(op, p, p->kernel_modules[i].module_filename,
-                                    insmod_ret);
-            if (ret) {
+        if (insmod_ret == EEXIST) {
+            /* The kernel module was already loaded: propagate this error back
+             * to test_kernel_modules() */
+            break;
+        } else if (insmod_ret != 0) {
+            const char *name = p->kernel_modules[i].module_filename;
+            int ignore = ignore_load_error(op, p, name, insmod_ret);
+
+            if (ignore) {
                 op->skip_module_load = TRUE;
-                ui_log(op, "Ignoring failure to load %s.",
-                       p->kernel_modules[i].module_filename);
+                ui_log(op, "Ignoring failure to load %s.", name);
+                insmod_ret = 0;
             } else {
                 handle_optional_module_failure(op, p->kernel_modules[i],
                                                "load");
             }
-        }
 
-        if (!module_success) {
-            goto test_exit;
+            break;
         }
     }
 
@@ -1399,34 +1525,59 @@ int test_kernel_modules(Options *op, Package *p)
 
     check_for_warning_messages(op);
 
+    if (pause_udev) {
+        /*
+         * Re-enable the udev event queue, if we disabled it. This is done
+         * before unloading the modules so that udev only has to process a
+         * backlog of events trigered by loading the modules, without also
+         * having to process a backlog of events triggered by unloading them.
+         */
+        toggle_udev_event_queue(op, TRUE);
+    }
+
     unload_kernel_modules(op, p);
 
-    ret = TRUE;
-
-test_exit:
-    /*
-     * display/log the last few lines of the kernel ring buffer
-     * to provide further details in case of a load failure or
-     * to capture NVRM warning messages, if any.
-     */
-    cmd = nvstrcat(op->utils[DMESG], " | ",
-                   op->utils[TAIL], " -n 25", NULL);
-
-    if (!run_command(op, cmd, &data, FALSE, 0, TRUE))
-        ui_log(op, "Kernel messages:\n%s", data);
-
-    nvfree(cmd);
-    nvfree(data);
+    log_dmesg(op);
 
     /*
      * Unload dependencies that might have been loaded earlier.
      */
 
     for (i = 0; i < ARRAY_LEN(depmods); i++) {
-        modprobe_remove_kernel_module_quiet(op, depmods[i]);
+        if (depmods[i]) {
+            modprobe_remove_kernel_module_quiet(op, depmods[i]);
+        }
     }
 
-    return ret;
+    return insmod_ret;
+}
+
+
+
+/*
+ * test_kernel_modules() - attempt to insmod the kernel modules and then rmmod
+ * them.  Return TRUE if the insmod succeeded, or FALSE otherwise.
+ */
+
+int test_kernel_modules(Options *op, Package *p)
+{
+    int ret;
+
+    if (op->skip_module_load) {
+        return TRUE;
+    }
+
+    ret = test_kernel_modules_helper(op, p, FALSE);
+
+    if (ret == EEXIST) {
+        ui_log(op, "One or more kernel modules were already loaded before "
+               "the module test load; trying again with the udev event "
+               "queue paused.");
+
+        ret = test_kernel_modules_helper(op, p, TRUE);
+    }
+
+    return ret == 0;
 }
 
 
@@ -1458,7 +1609,7 @@ static int modprobe_helper(Options *op, const char *module_name,
 
     loglevel_set = set_loglevel(PRINTK_LOGLEVEL_KERN_ALERT, &old_loglevel);
 
-    ret = run_command(op, cmd, &data, FALSE, 0, TRUE);
+    ret = run_command(op, cmd, &data, FALSE, NULL, TRUE);
 
     if (loglevel_set) {
         set_loglevel(old_loglevel, NULL);
@@ -1472,6 +1623,8 @@ static int modprobe_helper(Options *op, const char *module_name,
                  unload ? "unload" : "load", module_name,
                  op->expert? expert_detail : ".");
         nvfree(expert_detail);
+
+	log_dmesg(op);
     }
 
     nvfree(data);
@@ -1510,23 +1663,23 @@ int check_for_unloaded_kernel_module(Options *op)
 
     /*
      * We can skip this check if we are installing for a non-running
-     * kernel and only installing a kernel module.
+     * kernel and only installing the kernel modules.
      */
 
-    if (op->kernel_module_only && op->kernel_name) {
-        ui_log(op, "Only installing a kernel module for a non-running "
+    if (op->kernel_modules_only && op->kernel_name) {
+        ui_log(op, "Only installing kernel modules for a non-running "
                "kernel; skipping the \"is an NVIDIA kernel module loaded?\" "
                "test.");
         return TRUE;
     }
 
     /*
-     * We can also skip this check if we aren't installing a kernel
-     * module at all.
+     * We can also skip this check if we aren't installing any kernel
+     * modules at all.
      */
 
-    if (op->no_kernel_module) {
-        ui_log(op, "Not installing a kernel module; skipping the \"is an "
+    if (op->no_kernel_modules) {
+        ui_log(op, "Not installing any kernel modules; skipping the \"is an "
                "NVIDIA kernel module loaded?\" test.");
         return TRUE;
     }
@@ -1901,6 +2054,14 @@ static char *default_kernel_source_path(Options *op)
 
         nvfree(str);
 
+        str = nvstrcat("/lib/modules/", tmp, "/build/source", NULL);
+
+        if (directory_exists(str)) {
+            return str;
+        }
+
+        nvfree(str);
+
         str = nvstrcat("/lib/modules/", tmp, "/build", NULL);
     
         if (directory_exists(str)) {
@@ -2016,7 +2177,7 @@ static int check_for_loaded_kernel_module(Options *op, const char *module_name)
     char *result = NULL;
     int ret, found = FALSE;
 
-    ret = run_command(op, op->utils[LSMOD], &result, FALSE, 0, TRUE);
+    ret = run_command(op, op->utils[LSMOD], &result, FALSE, NULL, TRUE);
     
     if ((ret == 0) && (result) && (result[0] != '\0')) {
         char *ptr;
@@ -2052,7 +2213,7 @@ int rmmod_kernel_module(Options *op, const char *module_name)
 
     loglevel_set = set_loglevel(PRINTK_LOGLEVEL_KERN_ALERT, &old_loglevel);
     
-    ret = run_command(op, cmd, NULL, FALSE, 0, TRUE);
+    ret = run_command(op, cmd, NULL, FALSE, NULL, TRUE);
 
     if (loglevel_set) {
         set_loglevel(old_loglevel, NULL);
@@ -2066,61 +2227,6 @@ int rmmod_kernel_module(Options *op, const char *module_name)
 
 
 
-
-/*
- * check_cc_version() - check if the selected or default system
- * compiler is compatible with the one that was used to build the
- * currently running kernel.
- */
-
-static int check_cc_version(Options *op, Package *p)
-{
-    char *result;
-    int ret;
-
-    /* 
-     * If we're building/installing for a different kernel, then we
-     * can't do the gcc version check (we don't have a /proc/version
-     * string from which to get the kernel's gcc version).
-     * If the user passes the option no-cc-version-check, then we also
-     * shouldn't perform the cc version check.
-     */
-
-    if (op->ignore_cc_version_check) {
-        setenv("IGNORE_CC_MISMATCH", "1", 1);
-        return TRUE;
-    }
-
-    ret = run_conftest(op, p->kernel_module_build_directory,
-                       "cc_version_check just_msg", &result);
-
-    if (!ret) {
-        const char *choices[2] = {
-            "Ignore CC version check",
-            "Abort installation"
-        };
-
-        ret = (ui_multiple_choice(op, choices, 2, 0,
-                                  "The CC version check failed:\n\n%s\n\n"
-                                  "This may lead to subtle problems; if you "
-                                  "are not certain whether the mismatched "
-                                  "compiler will be compatible with your "
-                                  "kernel, you may wish to abort installation, "
-                                  "set the CC environment variable to the name "
-                                  "of the compiler used to compile your kernel, "
-                                  "and restart installation.", result) == 0);
-
-        if (ret) {
-            setenv("IGNORE_CC_MISMATCH", "1", 1);
-            ui_warn(op, "Ignoring CC version mismatch:\n\n%s", result);
-        }
-    }
-
-    nvfree(result);
-    return ret;
-} /* check_cc_version() */
-
-
 /*
  * conftest_sanity_check() - run the given sanity check conftest; if the test
  * fails, print the error message from the test. Return the status from the
@@ -2266,74 +2372,58 @@ static char *convert_include_path_to_source_path(const char *inc)
 
 
 /*
- * get_machine_arch() - get the machine architecture, substituting
- * i386 for i586 and i686 or arm for arm7l.
+ * get_machine_arch() - get the machine architecture
  */
 
-static char __machine_arch[16];
-
-char *get_machine_arch(Options *op)
+const char *get_machine_arch(Options *op)
 {
-    struct utsname uname_buf;
+    static struct utsname uname_buf;
+
+    if (uname_buf.machine[0]) {
+        return uname_buf.machine;
+    }
 
     if (uname(&uname_buf) == -1) {
         ui_warn(op, "Unable to determine machine architecture (%s).",
                 strerror(errno));
+        uname_buf.machine[0] = '\0';
         return NULL;
     } else {
-        if ((strncmp(uname_buf.machine, "i586", 4) == 0) ||
-            (strncmp(uname_buf.machine, "i686", 4) == 0)) {
-            strcpy(__machine_arch, "i386");
-        } else if ((strncmp(uname_buf.machine, "armv", 4) == 0)) {
-            strcpy(__machine_arch, "arm");
-        } else {
-            strncpy(__machine_arch, uname_buf.machine,
-                    sizeof(__machine_arch));
-        }
-        return __machine_arch;
+        return uname_buf.machine;
     }
-
 } /* get_machine_arch() */
 
 /*
- * Run `make` with the specified target and make variables. The variables
- * are given in the form of a NULL-terminated array of alternating key
- * and value strings, e.g. { "KEY1", "value1", "KEY2", "value2", NULL }.
+ * Run `make` with the options we need for the kernel module build, plus
+ * any user-supplied command line options.
+ *
  * If a 'status' string is given, then a ui_status progress bar is shown
  * using 'status' as the initial message, expecting 'lines' lines of output
  * from the make command.
  */
-static int run_make(Options *op, Package *p, const char *dir, const char *target,
-                    char **vars, const char *status, int lines) {
+static int run_make(Options *op, Package *p, const char *dir,
+                    const char *cli_options, const char *status,
+                    const RunCommandOutputMatch *match) {
     char *cmd, *concurrency, *data = NULL;
-    int i = 0, ret;
+    int ret;
 
     concurrency = nvasprintf(" -j%d ", op->concurrency_level);
 
     cmd = nvstrcat("cd ", dir, "; ",
-                   op->utils[MAKE], " -k", concurrency, target,
+                   op->utils[MAKE], " -k", concurrency,
                    " NV_EXCLUDE_KERNEL_MODULES=\"",
                    p->excluded_kernel_modules, "\"",
                    " SYSSRC=\"", op->kernel_source_path, "\"",
-                   " SYSOUT=\"", op->kernel_output_path, "\"",
+                   " SYSOUT=\"", op->kernel_output_path, "\" ",
+                   cli_options,
                    NULL);
     nvfree(concurrency);
 
-    if (vars) {
-        while (vars[i] && vars[i+1]) {
-            char *old_cmd = cmd;
-
-            cmd = nvstrcat(old_cmd, " ", vars[i], "=\"", vars[i+1], "\"", NULL);
-            nvfree(old_cmd);
-            i += 2;
-        }
-    }
-
     if (status) {
         ui_status_begin(op, status, "");
     }
 
-    ret = (run_command(op, cmd, &data, TRUE, status ? lines : 0, TRUE) == 0);
+    ret = (run_command(op, cmd, &data, TRUE, status ? match : NULL, TRUE) == 0);
 
     if (status) {
         if (ret) {
@@ -2353,15 +2443,24 @@ static int run_make(Options *op, Package *p, const char *dir, const char *target
             status_extra = nvstrdup("");
         }
 
-        ui_error(op, "An error occurred%s. See " DEFAULT_LOG_FILE_NAME
-                     " for details.", status_extra);
+        ui_error(op, "An error occurred%s. See %s for details.",
+                 status_extra, op->log_file_name);
         ui_log(op, "The command `%s` failed with the following output:\n\n%s",
                cmd, data);
         nvfree(status_extra);
     }
 
+    /* Append the make output to the running make log */
+    if (p->kernel_make_logs) {
+        char *old_logs = p->kernel_make_logs;
+        p->kernel_make_logs = nvstrcat(old_logs, data, NULL);
+        nvfree(old_logs);
+        nvfree(data);
+    } else {
+        p->kernel_make_logs = data;
+    }
+
     nvfree(cmd);
-    nvfree(data);
 
     return ret;
 }
@@ -2480,7 +2579,7 @@ static char *get_cpu_type(const Options *op)
         char *line, *ret = NULL;
         int eof;
 
-        while((line = fget_next_line(fp, &eof))) {
+        while ((line = fget_next_line(fp, &eof))) {
             ret = nvrealloc(ret, strlen(line) + 1);
 
             if (sscanf(line, "cpu : %s", ret) == 1) {
