@@ -82,6 +82,7 @@ static const struct {
     { ENTRY(OPENCL_WRAPPER_SYMLINK,     T, F, T, T, F, F, F, F, F) },
     { ENTRY(TLS_LIB,                    T, T, T, F, T, T, F, T, F) },
     { ENTRY(UTILITY_LIB,                T, T, F, F, T, F, F, T, F) },
+    { ENTRY(GBM_BACKEND_LIB,            T, T, F, F, T, F, F, T, F) },
     { ENTRY(DOCUMENTATION,              F, T, T, F, F, F, F, T, F) },
     { ENTRY(APPLICATION_PROFILE,        F, T, T, F, F, F, F, T, F) },
     { ENTRY(MANPAGE,                    F, T, T, F, F, F, F, T, F) },
@@ -89,10 +90,12 @@ static const struct {
     { ENTRY(OPENGL_SYMLINK,             T, F, F, T, F, T, F, T, F) },
     { ENTRY(CUDA_SYMLINK,               T, F, T, T, F, F, F, T, F) },
     { ENTRY(UTILITY_LIB_SYMLINK,        T, F, F, T, F, F, F, T, F) },
+    { ENTRY(GBM_BACKEND_LIB_SYMLINK,    T, F, F, T, F, F, F, T, F) },
     { ENTRY(INSTALLER_BINARY,           F, T, F, F, F, F, F, T, F) },
     { ENTRY(UTILITY_BINARY,             F, T, F, F, F, F, F, T, F) },
     { ENTRY(UTILITY_BIN_SYMLINK,        F, F, F, T, F, F, F, T, F) },
     { ENTRY(DOT_DESKTOP,                F, T, T, F, F, F, T, T, F) },
+    { ENTRY(ICON,                       F, T, T, F, F, F, F, T, F) },
     { ENTRY(XMODULE_SHARED_LIB,         F, T, T, F, T, F, F, T, F) },
     { ENTRY(GLX_MODULE_SHARED_LIB,      F, T, T, F, T, T, F, T, F) },
     { ENTRY(GLX_MODULE_SYMLINK,         F, F, T, T, F, T, F, T, F) },
@@ -109,8 +112,6 @@ static const struct {
     { ENTRY(NVIDIA_MODPROBE,            F, T, T, F, F, F, F, T, F) },
     { ENTRY(NVIDIA_MODPROBE_MANPAGE,    F, T, T, F, F, F, F, T, F) },
     { ENTRY(MODULE_SIGNING_KEY,         F, T, F, F, F, F, T, T, F) },
-    { ENTRY(NVIFR_LIB,                  T, T, F, F, T, F, F, T, F) },
-    { ENTRY(NVIFR_LIB_SYMLINK,          T, F, F, T, F, F, F, T, F) },
     { ENTRY(XORG_OUTPUTCLASS_CONFIG,    F, T, F, F, F, F, F, T, F) },
     { ENTRY(DKMS_CONF,                  F, T, F, F, F, F, T, T, T) },
     { ENTRY(GLVND_LIB,                  T, T, F, F, T, T, F, T, F) },
@@ -124,7 +125,6 @@ static const struct {
     { ENTRY(EGL_EXTERNAL_PLATFORM_JSON, F, T, F, F, F, T, F, T, F) },
     { ENTRY(FLEXERA_LIB,                F, T, T, F, T, F, F, F, F) },
     { ENTRY(FLEXERA_LIB_SYMLINK,        F, F, T, T, F, F, F, F, F) },
-    { ENTRY(ALLOCATOR_JSON,             F, T, F, F, F, F, F, T, F) },
     { ENTRY(INTERNAL_UTILITY_BINARY,    T, T, F, F, F, F, F, T, F) },
     { ENTRY(INTERNAL_UTILITY_LIB,       T, T, F, F, T, F, F, T, F) },
     { ENTRY(INTERNAL_UTILITY_DATA,      F, T, F, F, F, F, F, T, F) },
@@ -132,6 +132,7 @@ static const struct {
     { ENTRY(SYSTEMD_UNIT,               F, T, F, F, F, F, F, F, F) },
     { ENTRY(SYSTEMD_UNIT_SYMLINK,       F, F, T, T, F, F, F, F, F) },
     { ENTRY(SYSTEMD_SLEEP_SCRIPT,       F, T, F, F, F, F, F, F, F) },
+    { ENTRY(OPENGL_DATA,                F, T, T, F, F, T, F, T, F) },
 };
 
 /*
@@ -198,6 +199,10 @@ void get_installable_file_type_list(
             continue;
         }
 
+        if (type == FILE_TYPE_KERNEL_MODULE && op->dkms_registered) {
+            continue;
+        }
+
         if (((type == FILE_TYPE_NVIDIA_MODPROBE) ||
              (type == FILE_TYPE_NVIDIA_MODPROBE_MANPAGE)) &&
             !op->nvidia_modprobe) {
@@ -236,3 +241,9 @@ void add_symlinks_to_file_type_list(PackageEntryFileTypeList *file_type_list)
         file_type_list->types[type] = 1;
     }
 }
+
+void remove_file_type_from_file_type_list(PackageEntryFileTypeList *list,
+                                          PackageEntryFileType type)
+{
+    list->types[type] = 0;
+}
