@@ -41,6 +41,7 @@ typedef struct _LEGACY_STRINGS {
  * This table describes how we should refer to legacy branches.
  */
 static const LEGACY_STRINGS LegacyStrings[] = {
+    { 8, "470.xx" },
     { 7, "390.xx" },
     { 6, "367.xx" },
     { 5, "340.xx" },
@@ -661,6 +662,61 @@ static const LEGACY_INFO LegacyList[] = {
     { 0x0F01, 0x0000, 0x0000,  7,      "GeForce GT 620" },
     { 0x0F02, 0x0000, 0x0000,  7,      "GeForce GT 730" },
     { 0x0F03, 0x0000, 0x0000,  7,      "GeForce GT 610" },
+    { 0x0FC6, 0x0000, 0x0000,  8,      "GeForce GTX 650" },
+    { 0x0FC8, 0x0000, 0x0000,  8,      "GeForce GT 740" },
+    { 0x0FC9, 0x0000, 0x0000,  8,      "GeForce GT 730" },
+    { 0x0FCD, 0x0000, 0x0000,  8,      "GeForce GT 755M" },
+    { 0x0FCE, 0x0000, 0x0000,  8,      "GeForce GT 640M LE" },
+    { 0x0FD1, 0x0000, 0x0000,  8,      "GeForce GT 650M" },
+    { 0x0FD2, 0x0000, 0x0000,  8,      "GeForce GT 640M" },
+    { 0x0FD2, 0x1028, 0x0595,  8,      "GeForce GT 640M LE" },
+    { 0x0FD2, 0x1028, 0x05B2,  8,      "GeForce GT 640M LE" },
+    { 0x0FD3, 0x0000, 0x0000,  8,      "GeForce GT 640M LE" },
+    { 0x0FD4, 0x0000, 0x0000,  8,      "GeForce GTX 660M" },
+    { 0x0FD5, 0x0000, 0x0000,  8,      "GeForce GT 650M" },
+    { 0x0FD8, 0x0000, 0x0000,  8,      "GeForce GT 640M" },
+    { 0x0FD9, 0x0000, 0x0000,  8,      "GeForce GT 645M" },
+    { 0x0FDF, 0x0000, 0x0000,  8,      "GeForce GT 740M" },
+    { 0x0FE0, 0x0000, 0x0000,  8,      "GeForce GTX 660M" },
+    { 0x0FE1, 0x0000, 0x0000,  8,      "GeForce GT 730M" },
+    { 0x0FE2, 0x0000, 0x0000,  8,      "GeForce GT 745M" },
+    { 0x0FE3, 0x103C, 0x2B16,  8,      "GeForce GT 745A" },
+    { 0x0FE3, 0x17AA, 0x3675,  8,      "GeForce GT 745A" },
+    { 0x0FE3, 0x0000, 0x0000,  8,      "GeForce GT 745M" },
+    { 0x0FE4, 0x0000, 0x0000,  8,      "GeForce GT 750M" },
+    { 0x0FE9, 0x0000, 0x0000,  8,      "GeForce GT 750M" },
+    { 0x0FEA, 0x0000, 0x0000,  8,      "GeForce GT 755M" },
+    { 0x0FEC, 0x0000, 0x0000,  8,      "GeForce 710A" },
+    { 0x0FED, 0x0000, 0x0000,  8,      "GeForce 820M" },
+    { 0x0FEE, 0x0000, 0x0000,  8,      "GeForce 810M" },
+    { 0x0FF6, 0x0000, 0x0000,  8,      "Quadro K1100M" },
+    { 0x0FF8, 0x0000, 0x0000,  8,      "Quadro K500M" },
+    { 0x0FF9, 0x0000, 0x0000,  8,      "Quadro K2000D" },
+    { 0x0FFA, 0x0000, 0x0000,  8,      "Quadro K600" },
+    { 0x0FFB, 0x0000, 0x0000,  8,      "Quadro K2000M" },
+    { 0x0FFC, 0x0000, 0x0000,  8,      "Quadro K1000M" },
+    { 0x0FFD, 0x0000, 0x0000,  8,      "NVS 510" },
+    { 0x0FFE, 0x0000, 0x0000,  8,      "Quadro K2000" },
+    { 0x0FFF, 0x0000, 0x0000,  8,      "Quadro 410" },
+    { 0x1001, 0x0000, 0x0000,  8,      "GeForce GTX TITAN Z" },
+    { 0x1004, 0x0000, 0x0000,  8,      "GeForce GTX 780" },
+    { 0x1005, 0x0000, 0x0000,  8,      "GeForce GTX TITAN" },
+    { 0x1007, 0x0000, 0x0000,  8,      "GeForce GTX 780" },
+    { 0x1008, 0x0000, 0x0000,  8,      "GeForce GTX 780 Ti" },
+    { 0x100A, 0x0000, 0x0000,  8,      "GeForce GTX 780 Ti" },
+    { 0x100C, 0x0000, 0x0000,  8,      "GeForce GTX TITAN Black" },
+    { 0x1021, 0x0000, 0x0000,  8,      "Tesla K20Xm" },
+    { 0x1022, 0x0000, 0x0000,  8,      "Tesla K20c" },
+    { 0x1023, 0x0000, 0x0000,  8,      "Tesla K40m" },
+    { 0x1024, 0x0000, 0x0000,  8,      "Tesla K40c" },
+    { 0x1026, 0x0000, 0x0000,  8,      "Tesla K20s" },
+    { 0x1027, 0x0000, 0x0000,  8,      "Tesla K40st" },
+    { 0x1028, 0x0000, 0x0000,  8,      "Tesla K20m" },
+    { 0x1029, 0x0000, 0x0000,  8,      "Tesla K40s" },
+    { 0x102A, 0x0000, 0x0000,  8,      "Tesla K40t" },
+    { 0x102D, 0x0000, 0x0000,  8,      "Tesla K80" },
+    { 0x103A, 0x0000, 0x0000,  8,      "Quadro K6000" },
+    { 0x103C, 0x0000, 0x0000,  8,      "Quadro K5200" },
     { 0x1040, 0x0000, 0x0000,  7,      "GeForce GT 520" },
     { 0x1042, 0x0000, 0x0000,  7,      "GeForce 510" },
     { 0x1048, 0x0000, 0x0000,  7,      "GeForce 605" },
@@ -1063,6 +1119,56 @@ static const LEGACY_INFO LegacyList[] = {
     { 0x1140, 0x1179, 0xFA59,  7,      "GeForce 710M" },
     { 0x1140, 0x1179, 0xFA88,  7,      "GeForce 710M" },
     { 0x1140, 0x1179, 0xFA89,  7,      "GeForce 710M" },
+    { 0x1180, 0x0000, 0x0000,  8,      "GeForce GTX 680" },
+    { 0x1183, 0x0000, 0x0000,  8,      "GeForce GTX 660 Ti" },
+    { 0x1184, 0x0000, 0x0000,  8,      "GeForce GTX 770" },
+    { 0x1185, 0x0000, 0x0000,  8,      "GeForce GTX 660" },
+    { 0x1185, 0x10DE, 0x106F,  8,      "GeForce GTX 760" },
+    { 0x1187, 0x0000, 0x0000,  8,      "GeForce GTX 760" },
+    { 0x1188, 0x0000, 0x0000,  8,      "GeForce GTX 690" },
+    { 0x1189, 0x0000, 0x0000,  8,      "GeForce GTX 670" },
+    { 0x1189, 0x10DE, 0x1074,  8,      "GeForce GTX 760 Ti OEM" },
+    { 0x118A, 0x0000, 0x0000,  8,      "GRID K520" },
+    { 0x118E, 0x0000, 0x0000,  8,      "GeForce GTX 760 (192-bit)" },
+    { 0x118F, 0x0000, 0x0000,  8,      "Tesla K10" },
+    { 0x1193, 0x0000, 0x0000,  8,      "GeForce GTX 760 Ti OEM" },
+    { 0x1194, 0x0000, 0x0000,  8,      "Tesla K8" },
+    { 0x1195, 0x0000, 0x0000,  8,      "GeForce GTX 660" },
+    { 0x1198, 0x0000, 0x0000,  8,      "GeForce GTX 880M" },
+    { 0x1199, 0x0000, 0x0000,  8,      "GeForce GTX 870M" },
+    { 0x1199, 0x1458, 0xD001,  8,      "GeForce GTX 760" },
+    { 0x119A, 0x0000, 0x0000,  8,      "GeForce GTX 860M" },
+    { 0x119D, 0x0000, 0x0000,  8,      "GeForce GTX 775M" },
+    { 0x119E, 0x0000, 0x0000,  8,      "GeForce GTX 780M" },
+    { 0x119F, 0x0000, 0x0000,  8,      "GeForce GTX 780M" },
+    { 0x11A0, 0x0000, 0x0000,  8,      "GeForce GTX 680M" },
+    { 0x11A1, 0x0000, 0x0000,  8,      "GeForce GTX 670MX" },
+    { 0x11A2, 0x0000, 0x0000,  8,      "GeForce GTX 675MX" },
+    { 0x11A3, 0x0000, 0x0000,  8,      "GeForce GTX 680MX" },
+    { 0x11A7, 0x0000, 0x0000,  8,      "GeForce GTX 675MX" },
+    { 0x11B4, 0x0000, 0x0000,  8,      "Quadro K4200" },
+    { 0x11B6, 0x0000, 0x0000,  8,      "Quadro K3100M" },
+    { 0x11B7, 0x0000, 0x0000,  8,      "Quadro K4100M" },
+    { 0x11B8, 0x0000, 0x0000,  8,      "Quadro K5100M" },
+    { 0x11BA, 0x0000, 0x0000,  8,      "Quadro K5000" },
+    { 0x11BC, 0x0000, 0x0000,  8,      "Quadro K5000M" },
+    { 0x11BD, 0x0000, 0x0000,  8,      "Quadro K4000M" },
+    { 0x11BE, 0x0000, 0x0000,  8,      "Quadro K3000M" },
+    { 0x11C0, 0x0000, 0x0000,  8,      "GeForce GTX 660" },
+    { 0x11C2, 0x0000, 0x0000,  8,      "GeForce GTX 650 Ti BOOST" },
+    { 0x11C3, 0x0000, 0x0000,  8,      "GeForce GTX 650 Ti" },
+    { 0x11C4, 0x0000, 0x0000,  8,      "GeForce GTX 645" },
+    { 0x11C5, 0x0000, 0x0000,  8,      "GeForce GT 740" },
+    { 0x11C6, 0x0000, 0x0000,  8,      "GeForce GTX 650 Ti" },
+    { 0x11C8, 0x0000, 0x0000,  8,      "GeForce GTX 650" },
+    { 0x11CB, 0x0000, 0x0000,  8,      "GeForce GT 740" },
+    { 0x11E0, 0x0000, 0x0000,  8,      "GeForce GTX 770M" },
+    { 0x11E1, 0x0000, 0x0000,  8,      "GeForce GTX 765M" },
+    { 0x11E2, 0x0000, 0x0000,  8,      "GeForce GTX 765M" },
+    { 0x11E3, 0x0000, 0x0000,  8,      "GeForce GTX 760M" },
+    { 0x11E3, 0x17AA, 0x3683,  8,      "GeForce GTX 760A" },
+    { 0x11FA, 0x0000, 0x0000,  8,      "Quadro K4000" },
+    { 0x11FC, 0x0000, 0x0000,  8,      "Quadro K2100M" },
     { 0x1200, 0x0000, 0x0000,  7,      "GeForce GTX 560 Ti" },
     { 0x1201, 0x0000, 0x0000,  7,      "GeForce GTX 560" },
     { 0x1203, 0x0000, 0x0000,  7,      "GeForce GTX 460 SE v2" },
@@ -1088,7 +1194,53 @@ static const LEGACY_INFO LegacyList[] = {
     { 0x124B, 0x0000, 0x0000,  7,      "GeForce GT 640" },
     { 0x124D, 0x0000, 0x0000,  7,      "GeForce GT 555M" },
     { 0x124D, 0x1462, 0x10CC,  7,      "GeForce GT 635M" },
-    { 0x1251, 0x0000, 0x0000,  7,      "GeForce GTX 560M" }
+    { 0x1251, 0x0000, 0x0000,  7,      "GeForce GTX 560M" },
+    { 0x1280, 0x0000, 0x0000,  8,      "GeForce GT 635" },
+    { 0x1281, 0x0000, 0x0000,  8,      "GeForce GT 710" },
+    { 0x1282, 0x0000, 0x0000,  8,      "GeForce GT 640" },
+    { 0x1284, 0x0000, 0x0000,  8,      "GeForce GT 630" },
+    { 0x1286, 0x0000, 0x0000,  8,      "GeForce GT 720" },
+    { 0x1287, 0x0000, 0x0000,  8,      "GeForce GT 730" },
+    { 0x1288, 0x0000, 0x0000,  8,      "GeForce GT 720" },
+    { 0x1289, 0x0000, 0x0000,  8,      "GeForce GT 710" },
+    { 0x128B, 0x0000, 0x0000,  8,      "GeForce GT 710" },
+    { 0x1290, 0x0000, 0x0000,  8,      "GeForce GT 730M" },
+    { 0x1290, 0x103C, 0x2AFA,  8,      "GeForce GT 730A" },
+    { 0x1291, 0x0000, 0x0000,  8,      "GeForce GT 735M" },
+    { 0x1292, 0x0000, 0x0000,  8,      "GeForce GT 740M" },
+    { 0x1292, 0x17AA, 0x3675,  8,      "GeForce GT 740A" },
+    { 0x1292, 0x17AA, 0x367C,  8,      "GeForce GT 740A" },
+    { 0x1292, 0x17AA, 0x3684,  8,      "GeForce GT 740A" },
+    { 0x1293, 0x0000, 0x0000,  8,      "GeForce GT 730M" },
+    { 0x1295, 0x0000, 0x0000,  8,      "GeForce 710M" },
+    { 0x1295, 0x103C, 0x2B0D,  8,      "GeForce 710A" },
+    { 0x1295, 0x103C, 0x2B0F,  8,      "GeForce 710A" },
+    { 0x1295, 0x103C, 0x2B20,  8,      "GeForce 810A" },
+    { 0x1295, 0x103C, 0x2B21,  8,      "GeForce 810A" },
+    { 0x1295, 0x17AA, 0x367A,  8,      "GeForce 805A" },
+    { 0x1295, 0x17AA, 0x367C,  8,      "GeForce 710A" },
+    { 0x1296, 0x0000, 0x0000,  8,      "GeForce 825M" },
+    { 0x1298, 0x0000, 0x0000,  8,      "GeForce GT 720M" },
+    { 0x1299, 0x0000, 0x0000,  8,      "GeForce 920M" },
+    { 0x1299, 0x1B0A, 0x01C6,  8,      "GeForce GT 730" },
+    { 0x1299, 0x17AA, 0x30BB,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x30DA,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x30DC,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x30DD,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x30DF,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x3117,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x361B,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x362D,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x362E,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x3630,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x3637,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x369B,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x36A7,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x36AF,  8,      "GeForce 920A" },
+    { 0x1299, 0x17AA, 0x36F0,  8,      "GeForce 920A" },
+    { 0x129A, 0x0000, 0x0000,  8,      "GeForce 910M" },
+    { 0x12B9, 0x0000, 0x0000,  8,      "Quadro K610M" },
+    { 0x12BA, 0x0000, 0x0000,  8,      "Quadro K510M" }
 };
 
 #endif /* __NV_LEGACY_H */
