@@ -135,7 +135,7 @@ static Options *load_default_options(void)
     op->sigwinch_workaround = TRUE;
     op->run_distro_scripts = TRUE;
     op->no_kernel_module_source = FALSE;
-    op->dkms = FALSE;
+    op->dkms = TRUE;
     op->check_for_alternate_installs = TRUE;
     op->install_uvm = TRUE;
     op->install_drm = TRUE;
@@ -180,6 +180,12 @@ static int assign_file_type_override(Options *op, char *optarg)
     return TRUE;
 }
 
+static int print_recommended_module_type_option(Options *op)
+{
+    /* On 535, proprietary modules is always the default */
+    nv_info_msg(NULL, "proprietary");
+    return TRUE;
+}
 
 /*
  * parse_commandline() - Populate the Options structure with
@@ -194,6 +200,7 @@ static void parse_commandline(int argc, char *argv[], Options *op)
 {
     int c;
     int print_help_after = FALSE;
+    int print_recommended_kernel_module_after = FALSE;
     int print_help_args_only_after = FALSE;
     int print_advanced_help = FALSE;
     char *strval = NULL, *program_name = NULL;
@@ -239,8 +246,8 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case 'q': op->no_questions = TRUE; break;
         case 'b': op->no_backup = TRUE; break;
         case 'K':
-            op->kernel_module_only = TRUE;
-            op->no_kernel_module = FALSE; /* conflicts  */
+            op->kernel_modules_only = TRUE;
+            op->no_kernel_modules = FALSE; /* conflicts  */
             break;
         case 'X': op->run_nvidia_xconfig = TRUE; break;
         case 's':
@@ -252,7 +259,6 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case 'k':
             op->kernel_name = strval;
             op->no_precompiled_interface = TRUE;
-            op->ignore_cc_version_check = TRUE;
             break;
         case 'j':
             if (intval < 1) {
@@ -303,6 +309,8 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             op->utility_prefix = strval; break;
         case UTILITY_LIBDIR_OPTION:
             op->utility_libdir = strval; break;
+        case XDG_DATA_DIR_OPTION:
+            op->xdg_data_dir = strval; break;
         case KERNEL_SOURCE_PATH_OPTION:
             op->kernel_source_path = strval; break;
         case KERNEL_OUTPUT_PATH_OPTION:
@@ -388,16 +396,16 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case NO_SIGWINCH_WORKAROUND_OPTION:
             op->sigwinch_workaround = FALSE;
             break;
-        case NO_KERNEL_MODULE_OPTION:
-            op->no_kernel_module = TRUE;
-            op->kernel_module_only = FALSE; /* conflicts */
+        case NO_KERNEL_MODULES_OPTION:
+            op->no_kernel_modules = TRUE;
+            op->kernel_modules_only = FALSE; /* conflicts */
             op->no_kernel_module_source = TRUE;
             break;
         case NO_X_CHECK_OPTION:
             op->no_x_check = TRUE;
             break;
         case NO_CC_VERSION_CHECK_OPTION:
-            op->ignore_cc_version_check = TRUE;
+            /* This option is no longer used; ignore it. */
             break;
         case NO_DISTRO_SCRIPTS_OPTION:
             op->run_distro_scripts = FALSE;
@@ -418,7 +426,7 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             op->no_kernel_module_source = TRUE;
             break;
         case DKMS_OPTION:
-            op->dkms = TRUE;
+            op->dkms = boolval;
             break;
         case MODULE_SIGNING_SECRET_KEY_OPTION:
             op->module_signing_secret_key = strval;
@@ -519,6 +527,12 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case 'm':
             op->kernel_module_build_directory_override = strval;
             break;
+        case PRINT_RECOMMENDED_MODULE_TYPE_OPTION:
+            print_recommended_kernel_module_after = TRUE;
+            break;
+        case GBM_BACKEND_DIR_OPTION:
+            op->gbm_backend_dir = strval;
+            break;
         default:
             goto fail;
         }
@@ -542,18 +556,13 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         exit(0);
     }
 
-    /* Disable DKMS if module signing was requested; the two options are
-     * mutually exclusive. */
-    if (op->dkms &&
-        op->module_signing_secret_key && op->module_signing_public_key) {
-        ui_warn(op, "You selected the DKMS kernel module option, and also "
-                "specified a pair of module signing keys. Keys cannot be "
-                "managed securely under the automated DKMS framework. The "
-                "DKMS option will be disabled.");
-        op->dkms = FALSE;
+    if (print_recommended_kernel_module_after) {
+        if (!print_recommended_module_type_option(op)) {
+            goto fail;
+        }
+        exit(0);
     }
 
-
     /*
      * if the installer prefix was not specified, default it to the
      * utility prefix; this is done so that the installer prefix is
@@ -630,7 +639,9 @@ int main(int argc, char *argv[])
     
     /* check that we're running as root */
     
-    if (!check_euid(op)) goto done;
+    if (!op->add_this_kernel && !check_euid(op)) {
+        goto done;
+    }
     
     /*
      * find the system utilities we'll need
