@@ -37,13 +37,9 @@
 typedef enum {
     MIN_SYSTEM_UTILS = 0,
     LDCONFIG = MIN_SYSTEM_UTILS,
-    LDD,
     GREP,
     DMESG,
     TAIL,
-    CUT,
-    TR,
-    SED,
     MAX_SYSTEM_UTILS
 } SystemUtils;
 
@@ -59,6 +55,7 @@ typedef enum {
     OPENSSL,
     DKMS,
     SYSTEMCTL,
+    TAR,
     MAX_SYSTEM_OPTIONAL_UTILS
 } SystemOptionalUtils;
 
@@ -88,6 +85,10 @@ typedef enum {
     CC = MIN_DEVELOP_UTILS,
     MAKE,
     LD,
+    /* TR and SED aren't used directly by the installer, but conftest.sh
+     * depends on them, so we check for their presence */
+    TR,
+    SED,
     MAX_DEVELOP_UTILS
 } DevelopUtils;
 
@@ -110,6 +111,7 @@ typedef enum {
     FILE_TYPE_TLS_LIB,
     FILE_TYPE_UTILITY_LIB,
     FILE_TYPE_DOT_DESKTOP,
+    FILE_TYPE_ICON,
     FILE_TYPE_UTILITY_LIB_SYMLINK,
     FILE_TYPE_XMODULE_SHARED_LIB,
     FILE_TYPE_MANPAGE,
@@ -138,8 +140,6 @@ typedef enum {
     FILE_TYPE_NVIDIA_MODPROBE,
     FILE_TYPE_NVIDIA_MODPROBE_MANPAGE,
     FILE_TYPE_MODULE_SIGNING_KEY,
-    FILE_TYPE_NVIFR_LIB,
-    FILE_TYPE_NVIFR_LIB_SYMLINK,
     FILE_TYPE_XORG_OUTPUTCLASS_CONFIG,
     FILE_TYPE_DKMS_CONF,
     FILE_TYPE_GLVND_LIB,
@@ -153,7 +153,6 @@ typedef enum {
     FILE_TYPE_EGL_EXTERNAL_PLATFORM_JSON,
     FILE_TYPE_FLEXERA_LIB,
     FILE_TYPE_FLEXERA_LIB_SYMLINK,
-    FILE_TYPE_ALLOCATOR_JSON,
     FILE_TYPE_INTERNAL_UTILITY_BINARY,
     FILE_TYPE_INTERNAL_UTILITY_LIB,
     FILE_TYPE_INTERNAL_UTILITY_DATA,
@@ -161,6 +160,9 @@ typedef enum {
     FILE_TYPE_SYSTEMD_UNIT,
     FILE_TYPE_SYSTEMD_UNIT_SYMLINK,
     FILE_TYPE_SYSTEMD_SLEEP_SCRIPT,
+    FILE_TYPE_GBM_BACKEND_LIB,
+    FILE_TYPE_GBM_BACKEND_LIB_SYMLINK,
+    FILE_TYPE_OPENGL_DATA,
     FILE_TYPE_MAX
 } PackageEntryFileType;
 
@@ -194,8 +196,8 @@ typedef struct __options {
     int sanity;
     int add_this_kernel;
     int no_backup;
-    int kernel_module_only;
-    int no_kernel_module;
+    int kernel_modules_only;
+    int no_kernel_modules;
     int no_abi_note;
     int no_rpms;
     int no_recursion;
@@ -218,6 +220,7 @@ typedef struct __options {
     int install_peermem;
     int compat32_files_packaged;
     int x_files_packaged;
+    int vulkan_icd_json_packaged;
     int concurrency_level;
     int skip_module_load;
     int skip_depmod;
@@ -250,7 +253,10 @@ typedef struct __options {
     char *utility_bindir;
     char *installer_prefix;
 
-    char *dot_desktopdir;
+    char *gbm_backend_dir;
+
+    char *xdg_data_dir;
+    char *icon_dir;
 
     char *documentation_prefix;
     char *documentation_docdir;
@@ -291,11 +297,10 @@ typedef struct __options {
     char *external_platform_json_path;
 
     int kernel_module_signed;
+    int dkms_registered;
 
     void *ui_priv; /* for use by the ui's */
 
-    int ignore_cc_version_check;
-
     NVOptionalBool use_systemd;
     char *systemd_unit_prefix;
     char *systemd_sleep_prefix;
@@ -424,6 +429,7 @@ typedef struct __package {
     char *version;
     char *kernel_module_build_directory;
     char *precompiled_kernel_interface_directory;
+    char *kernel_make_logs;
 
     PackageEntry *entries; /* array of filename/checksum/bytesize entries */
     int num_entries;
@@ -464,6 +470,7 @@ typedef struct __package {
 #define DEFAULT_X_PREFIX                 "/usr/X11R6"
 #define DEFAULT_UTILITY_PREFIX           "/usr"
 #define DEFAULT_DOCUMENTATION_PREFIX     "/usr"
+#define DEFAULT_GBM_PREFIX               "/usr"
 #define DEFAULT_APPLICATION_PROFILE_PATH "/usr/share/nvidia"
 #define DEFAULT_WINE_PREFIX              "/usr"
 
@@ -479,7 +486,7 @@ typedef struct __package {
 #define DEFAULT_WINE_LIBDIR_SUFFIX      "nvidia/wine"
 #define DEFAULT_BINDIR                  "bin"
 #define DEFAULT_X_MODULEDIR             "modules"
-#define DEFAULT_DOT_DESKTOPDIR          "share/applications"
+#define DEFAULT_XDG_DATA_DIR            "share"
 #define DEFAULT_DOCDIR                  "share/doc"
 #define DEFAULT_MANDIR                  "share/man"
 #define DEFAULT_CONFDIR                 "X11/xorg.conf.d"
