@@ -56,6 +56,7 @@ enum {
     ADVANCED_OPTIONS_ARGS_ONLY_OPTION,
     UTILITY_PREFIX_OPTION,
     UTILITY_LIBDIR_OPTION,
+    XDG_DATA_DIR_OPTION,
     ADD_THIS_KERNEL_OPTION,
     RPM_FILE_LIST_OPTION,
     NO_RUNLEVEL_CHECK_OPTION,
@@ -78,7 +79,7 @@ enum {
     DOCUMENTATION_PREFIX_OPTION,
     APPLICATION_PROFILE_PATH_OPTION,
     X_LIBRARY_PATH_OPTION,
-    NO_KERNEL_MODULE_OPTION,
+    NO_KERNEL_MODULES_OPTION,
     NO_X_CHECK_OPTION,
     NO_CC_VERSION_CHECK_OPTION,
     NO_DISTRO_SCRIPTS_OPTION,
@@ -114,6 +115,8 @@ enum {
     SYSTEMD_UNIT_PREFIX_OPTION,
     SYSTEMD_SLEEP_PREFIX_OPTION,
     SYSTEMD_SYSCONF_PREFIX_OPTION,
+    GBM_BACKEND_DIR_OPTION,
+    PRINT_RECOMMENDED_MODULE_TYPE_OPTION,
 };
 
 static const NVGetoptOption __options[] = {
@@ -291,6 +294,19 @@ static const NVGetoptOption __options[] = {
       "' or '" DEFAULT_LIBDIR "' on 64bit " "systems, depending on the "
       "installed Linux distribution." },
 
+    { "xdg-data-dir", XDG_DATA_DIR_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
+      "The prefix under which XDG data files (such as application .desktop "
+      "files and icons) will be installed.  The default is '"
+      DEFAULT_UTILITY_PREFIX "/" DEFAULT_XDG_DATA_DIR "'." },
+
+    { "gbm-backend-dir", GBM_BACKEND_DIR_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
+      "The path relative to the OpenGL library installation prefix under "
+      "which the NVIDIA GBM backend library will be installed.  The "
+      "default is '$libdir/gbm', where '$libdir' is '" DEFAULT_64BIT_LIBDIR
+      "' or '" DEFAULT_LIBDIR "', depending on the installed Linux "
+      "distribution.  Only under very rare circumstances should this option be "
+      "used." },
+
     { "documentation-prefix", DOCUMENTATION_PREFIX_OPTION,
       NVGETOPT_STRING_ARGUMENT,  NULL,
       "The prefix under which the documentation files for the NVIDIA "
@@ -373,11 +389,13 @@ static const NVGetoptOption __options[] = {
       "Skip installation of 'nvidia-modprobe', a setuid root utility which "
       "nvidia-installer installs by default.  nvidia-modprobe can be used by "
       "user-space NVIDIA driver components to load the NVIDIA kernel module, "
-      "and create the NVIDIA device files, when those components run without "
-      "sufficient privileges to do so on their own, e.g., the CUDA driver run "
-      "within the permissions of a non-privileged user.  This utility is only "
-      "needed if other means of loading the NVIDIA kernel module and creating "
-      "the NVIDIA device files are unavailable." },
+      "create the NVIDIA device files, and configure certain runtime settings "
+      "in the kernel when those components run without sufficient privileges "
+      "to do so on their own, e.g., the CUDA driver run within the permissions "
+      "of a non-privileged user.  This utility is only needed if other means "
+      "of performing these privileged operations are unavailable. See the "
+      "nvidia-modprobe(1) man page for full details on the tasks performed by "
+      "this utility." },
 
     { "kernel-name", 'k', NVGETOPT_STRING_ARGUMENT, NULL,
       "Build and install the NVIDIA kernel module for the "
@@ -417,22 +435,30 @@ static const NVGetoptOption __options[] = {
       "and X server installation locations.  With this option set, "
       "the installer will only search in the top-level directories." },
 
+    /* alias for backwards compatibility */
     { "kernel-module-only", 'K',
+      NVGETOPT_OPTION_APPLIES_TO_NVIDIA_UNINSTALL, NULL, NULL },
+
+    { "kernel-modules-only", 'K',
       NVGETOPT_OPTION_APPLIES_TO_NVIDIA_UNINSTALL, NULL,
-      "Install a kernel module only, and do not uninstall the "
+      "Install the kernel modules only, and do not uninstall the "
       "existing driver.  This is intended to be used to install kernel "
       "modules for additional kernels (in cases where you might boot "
       "between several different kernels).  To use this option, you "
       "must already have a driver installed, and the version of the "
-      "installed driver must match the version of this kernel "
-      "module." },
+      "installed driver must match the version of these kernel "
+      "modules." },
 
-    { "no-kernel-module", NO_KERNEL_MODULE_OPTION, 0, NULL,
-      "Install everything but the kernel module, and do not remove any "
-      "existing, possibly conflicting kernel modules.  This can be "
-      "useful in some DEBUG environments.  If you use this option, you "
-      "must be careful to ensure that a NVIDIA kernel module matching "
-      "this driver version is installed separately." },
+    /* alias for backwards compatibility */
+    { "no-kernel-module", NO_KERNEL_MODULES_OPTION, 0, NULL, NULL },
+
+    { "no-kernel-modules", NO_KERNEL_MODULES_OPTION, 0, NULL,
+      "Install everything but the kernel modules, and do not remove any "
+      "existing, possibly conflicting, kernel modules.  This can be "
+      "useful in some debug environments, or when installing the open kernel "
+      "modules from source.  If you use this option, you "
+      "must be careful to ensure that NVIDIA kernel modules matching "
+      "this driver version are installed separately." },
 
     { "no-x-check", NO_X_CHECK_OPTION, 0, NULL,
       "Do not abort the installation if nvidia-installer detects that "
@@ -494,17 +520,6 @@ static const NVGetoptOption __options[] = {
       "restores the SIGWINCH signal handler after the child process "
       "has terminated.  This option disables this behavior." },
 
-    { "no-cc-version-check", NO_CC_VERSION_CHECK_OPTION, 0, NULL,
-      "The NVIDIA kernel module should be compiled with the same compiler that "
-      "was used to compile the currently running kernel. The layout of some "
-      "Linux kernel data structures may be dependent on the version of gcc "
-      "used to compile it. The Linux 2.6 kernel modules are tagged with "
-      "information about the compiler and the Linux kernel's module loader "
-      "performs a strict version match check. nvidia-installer checks for "
-      "mismatches prior to building the NVIDIA kernel module and aborts the "
-      "installation in case of failures. Use this option to override this "
-      "check." },
-
     { "no-distro-scripts", NO_DISTRO_SCRIPTS_OPTION,
       NVGETOPT_OPTION_APPLIES_TO_NVIDIA_UNINSTALL, NULL,
       "Normally, nvidia-installer will run scripts from /usr/lib/nvidia before "
@@ -530,16 +545,15 @@ static const NVGetoptOption __options[] = {
     { "no-kernel-module-source", NO_KERNEL_MODULE_SOURCE_OPTION, 0, NULL,
       "Skip installation of the kernel module source."},
 
-    { "dkms", DKMS_OPTION, 0, NULL,
+    { "dkms", DKMS_OPTION, NVGETOPT_IS_BOOLEAN, NULL,
       "nvidia-installer can optionally register the NVIDIA kernel module "
       "sources, if installed, with DKMS, then build and install a kernel "
       "module using the DKMS-registered sources.  This will allow the DKMS "
       "infrastructure to automatically build a new kernel module when "
       "changing kernels.  During installation, if DKMS is detected, "
       "nvidia-installer will ask the user if they wish to register the "
-      "module with DKMS; the default response is 'no'.  This option will "
-      "bypass the detection of DKMS, and cause the installer to attempt a "
-      "DKMS-based installation regardless of whether DKMS is present."},
+      "module with DKMS; the default response is 'yes' unless the --no-dkms "
+      "option is set, in which case the default response is 'no'." },
 
     { "module-signing-secret-key", MODULE_SIGNING_SECRET_KEY_OPTION, 
       NVGETOPT_STRING_ARGUMENT, NULL,
@@ -701,6 +715,13 @@ static const NVGetoptOption __options[] = {
       "build files is normally 'kernel/'.  This option can be used to "
       "override this value." },
 
+    { "print-recommended-kernel-module-type",
+      PRINT_RECOMMENDED_MODULE_TYPE_OPTION,  0, NULL,
+      "Print if \"open\" or \"proprietary\" modules should be used based on "
+      "currently attached GPUs and exit. NOTE: A kernel module type may be "
+      "recommended even if it is not included in this driver package."
+    },
+
     /* Orphaned options: These options were in the long_options table in
      * nvidia-installer.c but not in the help. */
     { "debug",                    'd', 0, NULL,NULL },
@@ -723,6 +744,7 @@ static const NVGetoptOption __options[] = {
       NULL, NULL },
     { "glvnd-glx-client", GLVND_GLX_CLIENT_OPTION, NVGETOPT_IS_BOOLEAN, NULL, NULL },
     { "glvnd-egl-client", GLVND_EGL_CLIENT_OPTION, NVGETOPT_IS_BOOLEAN, NULL, NULL },
+    { "no-cc-version-check", NO_CC_VERSION_CHECK_OPTION, 0, NULL, NULL },
 
     { NULL, 0, 0, NULL, NULL },
 };
